/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include "pan.h"

/*
 * globals 
 */

int group_mode = GROUP_ALL;
GtkWidget *groups_clist_menu;

GtkStyle *glist_font;

/* 
 * callback function 
 */
void
click_groups_column_cb (GtkCList *clist, int n)
{
	if (n)
		return;
	switch (group_mode) {
	case GROUP_SUBSCRIBED:
		group_mode = GROUP_NEW;
		break;
	case GROUP_NEW:
		group_mode = GROUP_ALL;
		break;
	case GROUP_ALL:
		group_mode = GROUP_SUBSCRIBED;
		break;
	}
	grouplist_update ();
}


void
group_set_attrib_string (group_data *gdata, char *attrib, char *value)
{
	if ( gdata->name )
	{
		gchar* section = g_strdup_printf ( "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib );
		gnome_config_set_string ( section, value );
		g_free ( section );
		gnome_config_sync ();
	}
}


gchar*
group_get_attrib_string (group_data *gdata, char *attrib)
{
	if ( !gdata->name )
		return NULL;
	else {
		gchar* section = g_strdup_printf ( "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib );
		gchar* retval = gnome_config_get_string ( section );
		g_free ( section );
		return retval;
	}
}


/*--------------------------------------------------------------------
 * easily set the ~/.gnome/Pan sections for [alt.group.name]
 *--------------------------------------------------------------------*/
void
group_set_attrib_i (group_data *gdata, char *attrib, int value)
{
	if ( gdata && gdata->name && attrib )
	{
		gchar* section = g_strdup_printf ( "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib );
		gnome_config_set_int ( section, value );
		g_free ( section );
		gnome_config_sync ();
	}
}


int
group_get_attrib_i (group_data *gdata, char *attrib)
{
	if (gdata && gdata->name && attrib)
	{
		gchar* section = g_strdup_printf ( "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib );
		int retval = gnome_config_get_int ( section );
		g_free ( section );
		return retval;
	}
	else return 0;
}


void
group_add_attrib_i (group_data *gdata, char *attrib, int i)
{
	int j;

	pan_lock();
	j = group_get_attrib_i (gdata, attrib);
	group_set_attrib_i (gdata, attrib, j + i);
	pan_unlock();

	/* Make the change visible */
	grouplist_update_row (gdata);
}


gint
compare_group_data (group_data *a, group_data *b)
{
	return strcmp(a->name, b->name);
}


void
check_new_group_list_for_newness ( GSList* list )
{
	/* assumes list and Pan.grouplist are sorted by name.. */
	GSList* a = list;
	GSList* b = Pan.grouplist;
	while ( a!=NULL && b!=NULL )
	{
		group_data* group_a = (group_data*) a->data;
		const group_data* group_b = (group_data*) b->data;
		const int rel = strcmp ( group_a->name, group_b->name );

		if ( rel>0 ) { /* old group not in new list */
		   b = b->next;
		   continue;
		}

		/* group_a is new if it's actually new or just
		   matches an old group still flagged as new... */
		if ( rel!=0 || (group_b->flags&GROUP_NEW) )
			group_a->flags |= GROUP_NEW;
		else
			group_a->flags &= ~GROUP_NEW;

		a = a->next;

		if ( !rel )
			b = b->next;
	}
	while ( a!=NULL ) /* exhausted Pan.grouplist, so all these are new... */
	{
	   ((group_data*)a->data)->flags |= GROUP_NEW;
	   a = a->next;
	}
}


static void
grouplist_init_styles (void)
{
	glist_font = gtk_style_copy (gtk_widget_get_style (GTK_WIDGET (Pan.window)));

	glist_font->font = gdk_font_load (grouplist_font);
}


void
add_group_to_clist (group_data *gdata)
{
	int row, i, j;
	char *text[4];
	GtkCListRow *clistrow;
	char *p = NULL;
	char *q = NULL;

	if ( grouplist_font && !glist_font)
		grouplist_init_styles ();

	if ((i = group_get_attrib_i (gdata, "Total"))) {
		q = g_strdup_printf ("%d", i);
		if ((j = group_get_attrib_i (gdata, "Read")))
			p = g_strdup_printf ("%d", i-j);
	}
	
	text[0] = gdata->name;
	text[1] = p;
	text[2] = q;
	text[3] = NULL;

	row = gtk_clist_append (GTK_CLIST(Pan.group_clist), text);
	g_free (p);
	g_free (q);

	if ( grouplist_font )
		gtk_clist_set_row_style (GTK_CLIST (Pan.group_clist), row, glist_font);

	if ( gdata->flags & GROUP_ONDISK ) {
		gtk_clist_set_pixtext (GTK_CLIST(Pan.group_clist), row, 0, gdata->name, 5,
				       disk_pixmap->pixmap, disk_pixmap->mask);
	}

	/* The following hack is *MUCH* faster than this:
	 * gtk_clist_set_row_data(GTK_CLIST(Pan.group_clist), row, gdata);
	 * Let's hope the gtk structure doesn't change or something
	 */
	clistrow = (GtkCListRow *) GTK_CLIST(Pan.group_clist)->row_list_end->data;
	clistrow->data = gdata;
}


void
add_subscribed_group_to_clist (group_data *gdata)
{
	if (gdata->flags & GROUP_SUBSCRIBED)
		add_group_to_clist (gdata);
}


void
add_new_group_to_clist (group_data *gdata)
{
	if (gdata->flags & GROUP_NEW)
		add_group_to_clist (gdata);
}


void
group_dialog_clicked_cb (GnomeDialog* dialog, int num)
{
	switch (num) {
	case 0:					/* OK button */
		queue_add_grouplist_download ();
	default:
		gtk_widget_destroy (GTK_WIDGET(dialog));
		break;
	}
}


void
group_free (group_data *gdata)
{
	g_free (gdata);
}


void
get_all_groups (void)
{
	queue_add_grouplist_download ();
}


void
choose_download_dialog_clicked_cb (GnomeDialog *dialog,
				   int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* All Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_ALL |
						QUEUE_FOREGROUND);
		break;
	case 1:					/* New Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE |
						QUEUE_FOREGROUND);
		break;
	default:
		/* Cancel */
		break;
	};
	gtk_widget_destroy (GTK_WIDGET(dialog));
}


void
choose_download_dialog (group_data *gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	char *temp;

	dialog = gnome_dialog_new (_("Select Header Download Option"),
				   _("Download All Headers"),
				   _("Sample Some Headers"),
				   GNOME_STOCK_BUTTON_CANCEL, NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (choose_download_dialog_clicked_cb),
			    gdata);

	temp = g_strdup_printf (_("Select an option for group: %s"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);

	gui_popup_draw (dialog);
}


void
group_unload (group_data *gdata)
{
	if (gdata == Pan.active_group) {
		articlelist_clear ();
		gui_set_title (NULL);
		Pan.active_group = NULL;
	}

}


/*---[ group_empty ]--------------------------------------------------
 * actual function that empties the group and deletes the groups
 * database file and associated gnome_config sections
 *--------------------------------------------------------------------*/
void
group_empty (group_data *gdata)
{
	if ( gdata )
	{
		gchar* section = g_strdup_printf ( "/Pan/%s:%s", Pan.active_server->name, gdata->name );

		group_unload (gdata);

		if (gnome_config_has_section (section)) {
			gnome_config_clean_section (section);
			gnome_config_sync();
		}

		g_free ( section );

		db_group_index_erase (gdata);

		if (gdata->flags & GROUP_SUBSCRIBED)
			gdata->flags &= ~GROUP_ONDISK;
		else 
			gdata->flags = 0;

		articlelist_free (gdata->articlelist);

		gdata->articlelist = NULL;
	}
}


/*--------------------------------------------------------------------
 * callback for group_empty, called when a button in group_empty_dialog
 * is clicked
 *--------------------------------------------------------------------*/
void
group_empty_cb (GnomeDialog *dialog,
		int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* 'yes' */
		group_empty (gdata);
		break;
	case 1:					/* 'no' */
		break;
	default:				/* cancel */
		break;
	};

//	if (group_mode == GROUP_SUBSCRIBED)
//		grouplist_update ();
//	else
//		grouplist_update_row (gdata);
	if (group_mode == GROUP_SUBSCRIBED)
		grouplist_update ();

	gtk_widget_destroy (GTK_WIDGET(dialog));
}


/*--------------------------------------------------------------------
 * dialog popup asking user if they'd like to remove all the messages
 * for a given group.  called when the user tries to unsubscribe
 * or they choose the menu option to empty the group manually
 *--------------------------------------------------------------------*/
void
group_empty_dialog (group_data *gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	gchar *temp;

	pan_lock ();
//	if (group_get_attrib_i (gdata, "Total") <= 0) {
//		group_unload (gdata);
//		if (group_mode == GROUP_SUBSCRIBED)
//			grouplist_update();
//		pan_unlock();
//		return;
//	}

	dialog = gnome_dialog_new (_("Clean group"),
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO,
				   NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_empty_cb),
			    gdata);

	temp = g_strdup_printf (_("Remove all messages from group: \n\n%s ?"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
	gui_popup_draw (dialog);
	pan_unlock ();
}


void
group_clean (group_data *gdata)
{
	articlelist_free (gdata->articlelist);
	gdata->articlelist = NULL;
}



/*
void
group_save (group_data *gdata, FILE *fp)
{
	fprintf (fp, "%d\n%s\n", gdata->flags, gdata->name);
}
*/
